#include "prettytasks.h"

#define PI 3.141592653

K_EXPORT_PLASMA_APPLET(pretty_tasks, PrettyTasks)

class PrettyTasks::Private
{
    public:
        Private()
            : scaledIconSize(64),
            cacheSize(0),
            dialog(0),
            layout(0)
        {};
        int scaledIconSize;
        int baseIconSize;
        int range;
        int cacheSize;
        bool onlyCurrentDesktop;
        PrettyLayout* layout;
        QHash<TaskPtr, PrettyIcon*> tasks;
        KDialog* dialog;
        Ui::prettyTasksConfig ui;
};

PrettyTasks::PrettyTasks(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    d(new Private())
{
    setDrawStandardBackground(true);
    //setAspectRatioMode(Qt::KeepAspectRatio);
    setAcceptsHoverEvents(true);
    setHasConfigurationInterface(true);
}
 
 
PrettyTasks::~PrettyTasks()
{
    QPixmapCache::setCacheLimit(QPixmapCache::cacheLimit() - d->cacheSize);
    delete d;
}
 
void PrettyTasks::init()
{
    d->layout = new PrettyLayout(this);
    d->layout->setMargin(0);
    reloadConfig();
    reload();
    // listen for addition and removal of window tasks
    connect(TaskManager::TaskManager::self(), SIGNAL(taskAdded(TaskPtr)),
            this, SLOT(addWindowTask(TaskPtr)));
    connect(TaskManager::TaskManager::self(), SIGNAL(taskRemoved(TaskPtr)),
            this, SLOT(removeWindowTask(TaskPtr)));
    if(d->onlyCurrentDesktop) {
        connect(TaskManager::TaskManager::self(), SIGNAL(desktopChanged(int)),
                this, SLOT(loadTasksForDesktop(int)));
    }
}

void PrettyTasks::setScaledIconSize(int size)
{
    d->scaledIconSize = size;
    reload();
}

void PrettyTasks::writeConfig()
{
    KConfigGroup cg = config();
    cg.writeEntry("onlyCurrentDesktop", (d->ui.onlyCurrentDesktop->checkState() == Qt::Checked));
    cg.writeEntry("iconSize", d->ui.iconSize->value());
    cg.writeEntry("range", d->ui.range->value());
    cg.writeEntry("cacheSize", d->ui.cacheSize->value());
    emit configNeedsSaving();
    reloadConfig();
    reload();
}

void PrettyTasks::showConfigurationInterface()
{
    if (d->dialog == 0) {
        d->dialog = new KDialog;
        d->dialog->setCaption(i18n("Configure Pretty Tasks"));

        QWidget *widget = new QWidget;
        d->ui.setupUi(widget);
        d->dialog->setMainWidget(widget);
        d->dialog->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);

        connect(d->dialog, SIGNAL(applyClicked()), this, SLOT(writeConfig()));
        connect(d->dialog, SIGNAL(okClicked()), this, SLOT(writeConfig()));
    }
    d->ui.onlyCurrentDesktop->setChecked(d->onlyCurrentDesktop ? Qt::Checked : Qt::Unchecked);
    d->ui.iconSize->setValue(d->baseIconSize);
    d->ui.range->setValue(d->range);
    d->ui.cacheSize->setValue(d->cacheSize);
    d->dialog->show();
}

void PrettyTasks::hoverLeaveEvent(QGraphicsSceneHoverEvent* e)
{
    for(int i = 0; i < d->layout->count(); i++) {
        dynamic_cast<PrettyIcon*>(d->layout->itemAt(i))->scaleTo(d->baseIconSize);
    }
    d->layout->updateGeometry();
    Plasma::Applet::hoverLeaveEvent(e);
}

void PrettyTasks::reloadConfig()
{
    KConfigGroup cg = config();
    d->onlyCurrentDesktop = cg.readEntry("onlyCurrentDesktop", false);
    d->baseIconSize = cg.readEntry("iconSize", 32);
    d->range = cg.readEntry("range", 3);
    kDebug() << d->range;
    int newCacheSize = cg.readEntry("cacheSize", 2048);
    QPixmapCache::setCacheLimit(QPixmapCache::cacheLimit() + newCacheSize - d->cacheSize);
    d->cacheSize = newCacheSize;
}

void PrettyTasks::reload()
{
    qDeleteAll(d->tasks);
    
    d->tasks.clear();
    TaskManager::TaskManager *manager = TaskManager::TaskManager::self();

    TaskManager::TaskDict tasks = manager->tasks();
    QMapIterator<WId, TaskPtr> iter(tasks);

    while(iter.hasNext()) {
        iter.next();
        addWindowTask(iter.value());
    }
}

void PrettyTasks::updateIconSizes(PrettyIcon* active, float across)
{
    int activeItemIndex = d->layout->indexOf(active);
    for(int i = 0; i < d->layout->count(); i++) {
        dynamic_cast<PrettyIcon*>(d->layout->itemAt(i))->scaleTo(PrettyTasks::calculateSize(i, activeItemIndex, across));
    }
    d->layout->updateGeometry();
}

int PrettyTasks::calculateSize(int position, int activeItemPosition, float across)
{
    //kDebug() << across;
    if(position < activeItemPosition - d->range || position > activeItemPosition + d->range) {
        return d->baseIconSize;
    } else if(position == activeItemPosition) {
        return d->scaledIconSize;
    } else if(position < activeItemPosition) {
        return d->baseIconSize + (d->scaledIconSize - d->baseIconSize - 1) * (std::cos((position - activeItemPosition - across + 1) / d->range * PI) + 1) / 2;
    } else {
        return d->baseIconSize + (d->scaledIconSize - d->baseIconSize - 1) * (std::cos((position - activeItemPosition - across) / d->range * PI) + 1) / 2;
    }
}

void PrettyTasks::addWindowTask(TaskPtr task)
{
    if(task->isOnCurrentDesktop() || !d->onlyCurrentDesktop) {
        PrettyIcon* icon = new PrettyIcon(task, d->scaledIconSize, d->baseIconSize, this);
        d->layout->addItem(icon);
        connect(icon, SIGNAL(gotHover(PrettyIcon*, float)), this, SLOT(updateIconSizes(PrettyIcon*, float)));
        d->tasks[task] = icon;
        updateGeometry();
    }
}

void PrettyTasks::removeWindowTask(TaskPtr task)
{
    PrettyIcon* icon = d->tasks[task];
    d->layout->removeItem(icon);
    d->tasks.remove(task);
    delete icon;
}

void PrettyTasks::loadTasksForDesktop(int desktop)
{
    Q_UNUSED(desktop);
    if(!d->onlyCurrentDesktop) {
        return;
    }
    reload();
}

#include "prettytasks.moc"
